<?php

namespace Teknisa\DF\Controllers;

use Estoque\Entities\Lanctoestoq;
use Fiscal\Entities\Notafiscal;
use Teknisa\DF\Services\Dfe57000LancamentoSimplificadoService;
use Teknisa\DF\Services\ValidacoesEstoqueService;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;

class Dfe57000LancamentoSimplificadoController extends \Teknisa\DF\Controllers\TeknisaCrud
{
    protected $Dfe57000LancamentoSimplificadoService;
    protected $validacoesEstoqueService;
    protected $nfEntradaService;

    public function __construct($crudService, $parameterBag, Dfe57000LancamentoSimplificadoService $Dfe57000LancamentoSimplificadoService, ValidacoesEstoqueService $validacoesEstoqueService)
    {
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->Dfe57000LancamentoSimplificadoService = $Dfe57000LancamentoSimplificadoService;
        $this->validacoesEstoqueService = $validacoesEstoqueService;
    }

    public function montaDatasourceNotas (Request\Filter $request, Response $response) {
    	$requestConditions = $request->getFilterCriteria()->getConditions();
        $filial = $requestConditions[0]["value"];
        $fornecedor = $requestConditions[1]["value"];
        $dtemissao = $requestConditions[2]["value"];
        
        $arrayNotas = $this->Dfe57000LancamentoSimplificadoService->buscaNotas($filial, $fornecedor, $dtemissao);

    	$response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('lancamento_simplificado', $arrayNotas));
    }

    public function excluirNotaSaida(Request\Filter $request, Response $response){
        try {
            $conditions = $request->getFilterCriteria()->getConditions();

            $filial = $conditions[0]['value'];
            $lancamento = $conditions[1]['value'];
            
            $filter = array(
                array(
                    "columnName" => "IDDOCREPLIC",
                    "operator"=> "=",
                    "value"=> $filial.$lancamento
                )  
            );
            
            $notasReplicadas = $this->crudService->getFilteredDataSet('notaReplicada_tipooperac', $filter);
            $notasReplicadas = $notasReplicadas->getRows();
            
            if($notasReplicadas) {
                foreach ($notasReplicadas as $notaReplic) {
                    $this->lancamentoNotafiscalService->procedureDeletaNf($notaReplic['CDFILIAL'], $notaReplic['NRLANCTONF']);
                }
            }
            $this->lancamentoNotafiscalService->validaExclusaoNotaUtilizaAf($filial, $lancamento);

            $lanctoestoq = Lanctoestoq::findOneByNrlanctonfAndCdfilial($lancamento, $filial);

            $this->lancamentoNotafiscalService->procedureDeletaNf($filial, $lancamento);

            if (is_object($lanctoestoq)) {
                $lancamentoEstoque = $lanctoestoq->getNrlancestq();
                $this->validacoesEstoqueService->setLanctoestoq($lanctoestoq);
                $this->validacoesEstoqueService->atualizaEstoque($filial, $lancamentoEstoque);
            }
            
            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('excluir_nota_saida', array('success' => 'S')));
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function salvarItem(Request\Filter $request, Response $response){
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $item = $conditions[0]['value'];

            if(!isset($item['CDALMOXARIFE'])) $item['CDALMOXARIFE'] = ' ';
            else if($item['CDALMOXARIFE'] == null) $item['CDALMOXARIFE'] = ' ';
            
            if(!isset($item['CDLOCALESTOQ'])) $item['CDLOCALESTOQ'] = ' ';
            else if($item['CDLOCALESTOQ'] == null) $item['CDLOCALESTOQ'] = ' ';
            
            if(!isset($item['NRLOTEESTQ']))   $item['NRLOTEESTQ']   = ' ';
            else if($item['NRLOTEESTQ'] == null) $item['NRLOTEESTQ'] = ' ';
            
            if(!isset($item['NRSUBLOTE']))    $item['NRSUBLOTE']    = ' ';
            else if($item['NRSUBLOTE'] == null) $item['NRSUBLOTE'] = ' ';
            
            if(empty($item['CDSERVFATU'])) $item['CDSERVFATU'] =  null;
            
            $this->lancamentoNotafiscalService->procedureGeraNotaFiscalProduto(
                $item['CDFILIAL'], $item['NRLANCTONF'], $item['CDPRODUTO'], $item['CDPRODEMIS'], $item['DSPRODEMIS'], $item['QTITEMPROD'], $item['VRUNITITEM'], $item['VRDESCITEM'], $item['CDALMOXARIFE'], $item['CDLOCALESTOQ'], $item['NRLOTEESTQ'], $item['NRSUBLOTE'], $item['CDSERVFATU']
            );

//            $this->alteraStatusNotafiscal($item['CDFILIAL'], $item['NRLANCTONF'], $item['NRORG'], 'D');
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function alteraStatusNotafiscal($filial, $lancamento, $organizacao, $status){
        //@todo somente quando status nota !== 'D'
        $this->lancamentoNotafiscalService->updateStatusNotafiscal($filial, $lancamento, $organizacao, $status);
    }

    public function fecharNota(Request\Filter $request, Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();

            $cabecalho = $conditions[0]['value'];
            $vencimentos = $conditions[1]['value'];

            $infoNotaFiscal = $this->lancamentoNotafiscalService->fechaNotaFiscal($cabecalho, $vencimentos);

            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('fecharNota', $infoNotaFiscal));
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function calculaImpostoItemProduto(Request\DataSet $request, Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            $cabecalho = $rows[0];
            $item = $rows[1];
            $calculoImpostoProduto = $this->lancamentoNotafiscalService->calculaImpostoItemProduto($cabecalho, $item);

            $response->addDataset(new DataSet('calculoImpostoProduto', $calculoImpostoProduto));
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function salvaItensAf(Request\DataSet $request, Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            $afSelecionadas = $rows;

            $arrayProdutos = $this->lancamentoNotafiscalService->salvaItensAf($afSelecionadas);

            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('lancamento_nota_itens', $arrayProdutos));
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function replicacaoManual(Request\Dataset $request, Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            $notafiscal = $rows[0];
            $documentos = $rows[1];
            $itens      = $rows[2];
            $replicacaoManual = $this->lancamentoNotafiscalService->replicacaoManual($notafiscal, $documentos, $itens);
            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('replicacaoManual', $replicacaoManual));
        }
        catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}