<?php

namespace Teknisa\DF\Modules\Saida\NFSe;

use Teknisa\DF\Modules\Saida\NFSe\Utilities\Util;
use Teknisa\Libs\Util\Environment;
use NFePHP\Common\Certificate;
use NFePHP\Common\Signer;

class NFSeContagem {
    const URL_PRODUCAO = "https://producao.ginfes.com.br/ServiceGinfesImpl?wsdl";
    const URL_HOMOLOGACAO = "https://homologacao.ginfes.com.br/ServiceGinfesImpl?wsdl";

    protected $connection;
    protected $environment;
    protected $newCode;
    
    protected $ambiente; //1 = producao, 2 = homologacao

    protected $xmlEnviarLoteRps;
    protected $listaRps = array();
    protected $listaAuxRps = array();
    protected $xmlConsultarLoteRps;
    protected $xmlConsultarSituacaoLoteRps;
    protected $xmlCancelarNota;

    protected $publicKey;
    protected $privateKeyId;
    protected $caminhoCertificado;
    protected $linkCertificado;
    protected $linkCertKey;
    protected $caminhoXml;

    protected $senhaCertificadoDigital;

    // Dados Prestador
    protected $InscricaoPrestador;
    protected $InscricaoMunicipalPrestador;
    protected $RazaoSocialPrestador;
    protected $CodigoMunicipioPrestador;

    // Dados Lote RPS
    protected $IdLote;
    protected $NumeroLote;

    // Dados RPS
    protected $IdRps;
    protected $NumeroRps;
    protected $SerieRps;
    protected $TipoRps;
    protected $DataEmissao;
    protected $NaturezaOperacao;
    protected $RegimeEspecialTributacao;
    protected $OptanteSimplesNacional;
    protected $IncentivadorCultural;
    protected $StatusRps;
    protected $ValorServicos;
    protected $ValorDeducoes;
    protected $ValorPis;
    protected $ValorCofins;
    protected $ValorInss;
    protected $ValorIr;
    protected $ValorCsll;
    protected $IssRetido;
    protected $ValorIss;
    protected $ValorIssRetido;
    protected $OutrasRetencoes;
    protected $BaseCalculo;
    protected $Aliquota;
    protected $ValorLiquidoNfse;
    protected $DescontoCondicionado;
    protected $DescontoIncondicionado;
    protected $ItemListaServico;
    protected $CodigoCnae;
    protected $CodigoTributacaoMunicipio;
    protected $Discriminacao;
    protected $CodigoMunicipioTomador;
    protected $InscricaoTomador;
    protected $InscricaoMunicipalTomador;
    protected $RazaoSocialTomador;
    protected $Endereco;
    protected $Numero;
    protected $Complemento;
    protected $Bairro;
    protected $Uf;
    protected $Cep;
    protected $Telefone;
    protected $Email;

    public function __construct($connection, Environment $environment, $newCode, $xmlDir, $certUploadDir = null, $certUploadLink = null, $certKeyLink = null) {
        $this->connection = $connection;
        $this->environment = $environment;
        $this->newCode = $newCode;
        
        //$this->setCaminhoXml($xmlDir);
        if($certUploadDir) {
            $this->setCaminhoCertificado($certUploadDir);
        }
        else {
            $this->linkCertificado = $certUploadLink;
            $this->linkCertKey     = $certKeyLink;
        }
    }
    
    private function setCaminhoXml($xmlDir) {
	    $xmlDir .= "org_". $this->environment->getNrOrg() . DIRECTORY_SEPARATOR;
	    $this->caminhoXml = $xmlDir;
	    if (!is_dir($this->caminhoXml)) {
            mkdir($this->caminhoXml, 0775, true);
            chmod($this->caminhoXml, 0775);
        }
	}

	private function setCaminhoCertificado($certUploadDir) {
	    if (!is_null($certUploadDir)) {
	        $certUploadDir .= "org_". $this->environment->getNrOrg() . DIRECTORY_SEPARATOR;
	        $this->caminhoCertificado = $certUploadDir;
	        if (!is_dir($this->caminhoCertificado)) {
	            mkdir($this->caminhoCertificado, 0775, true);
	            chmod($this->caminhoCertificado, 0775);
	        }
	    }
	}

    private function gerarXmlEnviarLoteRps() {
        $this->xmlEnviarLoteRps = '
        <EnviarLoteRpsEnvio xmlns="http://www.ginfes.com.br/servico_enviar_lote_rps_envio_v03.xsd" xmlns:tipos="http://www.ginfes.com.br/tipos_v03.xsd">
            <LoteRps Id="'.$this->IdLote.'">
                <tipos:NumeroLote>'.$this->NumeroLote.'</tipos:NumeroLote>
                <tipos:Cnpj>'.$this->InscricaoPrestador.'</tipos:Cnpj>
                <tipos:InscricaoMunicipal>'.$this->InscricaoMunicipalPrestador.'</tipos:InscricaoMunicipal>
                <tipos:QuantidadeRps>'.count($this->listaRps).'</tipos:QuantidadeRps>
                <tipos:ListaRps>';
                foreach ($this->listaRps as $rps) {
                    $this->xmlEnviarLoteRps .= "\n" . $rps;
                }

                $this->xmlEnviarLoteRps .= '
                </tipos:ListaRps>
            </LoteRps>
        </EnviarLoteRpsEnvio>';
    }

    private function adicionarRpsEnviarLote($cdfilial, $nrlanctonf) {
        $xmlRps = '
                <tipos:Rps xmlns:tipos="http://www.ginfes.com.br/tipos_v03.xsd">
                    <tipos:InfRps Id="'.$this->IdRps.'">
                        <tipos:IdentificacaoRps>
                            <tipos:Numero>'.$this->NumeroRps.'</tipos:Numero>
                            <tipos:Serie>'.$this->SerieRps.'</tipos:Serie>
                            <tipos:Tipo>'.$this->TipoRps.'</tipos:Tipo>
                        </tipos:IdentificacaoRps>
                        <tipos:DataEmissao>'.$this->DataEmissao.'</tipos:DataEmissao>
                        <tipos:NaturezaOperacao>'.$this->NaturezaOperacao.'</tipos:NaturezaOperacao>
                        <tipos:RegimeEspecialTributacao>'.$this->RegimeEspecialTributacao.'</tipos:RegimeEspecialTributacao>
                        <tipos:OptanteSimplesNacional>'.$this->OptanteSimplesNacional.'</tipos:OptanteSimplesNacional>
                        <tipos:IncentivadorCultural>'.$this->IncentivadorCultural.'</tipos:IncentivadorCultural>
                        <tipos:Status>'.$this->StatusRps.'</tipos:Status>';
                        
                    if($substituicao = Util::getNotafiscalSubstituicaoNfse($this->connection, $cdfilial, $nrlanctonf)) {
	                        $xmlRps .= 
                        '<tipos:RpsSubstituido>
	                        <tipos:Numero>'.$substituicao["NRRPS"].'</tipos:Numero>
						    <tipos:Serie>'.$substituicao["CDSERIERPS"].'</tipos:Serie>
						    <tipos:Tipo>1</tipos:Tipo> 
						</tipos:RpsSubstituido>';
	                    
	                };
                      $xmlRps .= '  
                        <tipos:Servico>
                            <tipos:Valores>
                                <tipos:ValorServicos>'.Util::formatarNumero($this->ValorServicos).'</tipos:ValorServicos>
                                <tipos:ValorDeducoes>'.Util::formatarNumero($this->ValorDeducoes).'</tipos:ValorDeducoes>
                                <tipos:ValorPis>'.Util::formatarNumero($this->ValorPis).'</tipos:ValorPis>
                                <tipos:ValorCofins>'.Util::formatarNumero($this->ValorCofins).'</tipos:ValorCofins>
                                <tipos:ValorInss>'.Util::formatarNumero($this->ValorInss).'</tipos:ValorInss>
                                <tipos:ValorIr>'.Util::formatarNumero($this->ValorIr).'</tipos:ValorIr>
                                <tipos:ValorCsll>'.Util::formatarNumero($this->ValorCsll).'</tipos:ValorCsll>
                                <tipos:IssRetido>'.$this->IssRetido.'</tipos:IssRetido>
                                <tipos:ValorIss>'.Util::formatarNumero($this->ValorIss).'</tipos:ValorIss>
                                <tipos:ValorIssRetido>'.Util::formatarNumero($this->ValorIssRetido).'</tipos:ValorIssRetido>
                                <tipos:BaseCalculo>'.Util::formatarNumero($this->BaseCalculo).'</tipos:BaseCalculo>
                                <tipos:Aliquota>'.Util::formatarNumero($this->Aliquota).'</tipos:Aliquota>
                                <tipos:ValorLiquidoNfse>'.Util::formatarNumero($this->ValorLiquidoNfse).'</tipos:ValorLiquidoNfse>
                            </tipos:Valores>
                            <tipos:ItemListaServico>'.$this->ItemListaServico.'</tipos:ItemListaServico>
                            <tipos:CodigoTributacaoMunicipio>'.$this->CodigoTributacaoMunicipio.'</tipos:CodigoTributacaoMunicipio>
                            <tipos:Discriminacao>'.$this->Discriminacao.'</tipos:Discriminacao>
                            <tipos:CodigoMunicipio>'.$this->CodigoMunicipioPrestador.'</tipos:CodigoMunicipio>
                        </tipos:Servico>
                        <tipos:Prestador>
                            <tipos:Cnpj>'.$this->InscricaoPrestador.'</tipos:Cnpj>
                            <tipos:InscricaoMunicipal>'.$this->InscricaoMunicipalPrestador.'</tipos:InscricaoMunicipal>
                        </tipos:Prestador>
                        <tipos:Tomador>
                            <tipos:IdentificacaoTomador>
                                <tipos:CpfCnpj>
                                    ';

                                if (strlen($this->InscricaoTomador) == 11) {
                                    $xmlRps .= '<tipos:Cpf>'.$this->InscricaoTomador.'</tipos:Cpf>';
                                }
                                else {
                                    $xmlRps .= '<tipos:Cnpj>'.$this->InscricaoTomador.'</tipos:Cnpj>';
                                }
                                
                                $xmlRps .= '
                                </tipos:CpfCnpj>
                            <tipos:InscricaoMunicipal>'.$this->InscricaoMunicipalTomador.'</tipos:InscricaoMunicipal>
                            </tipos:IdentificacaoTomador>
                            <tipos:RazaoSocial>'.$this->RazaoSocialTomador.'</tipos:RazaoSocial>
                            <tipos:Endereco>
                                <tipos:Endereco>'.$this->Endereco.'</tipos:Endereco>
                                <tipos:Numero>'.$this->Numero.'</tipos:Numero>
                                <tipos:Bairro>'.$this->Bairro.'</tipos:Bairro>
                                <tipos:CodigoMunicipio>'.$this->CodigoMunicipioTomador.'</tipos:CodigoMunicipio>
                                <tipos:Uf>'.$this->Uf.'</tipos:Uf>
                                <tipos:Cep>'.$this->Cep.'</tipos:Cep>
                            </tipos:Endereco>
                        </tipos:Tomador>
                    </tipos:InfRps>
                </tipos:Rps>';

        /*
        valores

        <ValorIssRetido>'.Util::formatarNumero($this->ValorIssRetido).'</ValorIssRetido>
        <DescontoCondicionado>'.Util::formatarNumero($this->DescontoCondicionado).'</DescontoCondicionado>
        <DescontoIncondicionado>'.Util::formatarNumero($this->DescontoIncondicionado).'</DescontoIncondicionado>
        */

        $this->listaRps[] = $xmlRps;
    }

    private function assinarXml($tags, &$xml) {
        $xml = str_replace("&", "&#38;",$xml);
        $domXml = new \DOMDocument("1.0", "UTF-8");
        $domXml->loadXML($xml, LIBXML_NOBLANKS);
        $domXml = $this->removerTagsVazias($domXml);
        $domXml->formatOutput = false;
        $xmlTemp = $domXml->saveXML($domXml->documentElement);

        $domXml = $this->assinaXml($xmlTemp, $tags);
        
        if ($domXml === false) {
            throw new \Exception("Certificado digital não encontrado!");
        }
        else {
            $this->limparListaRps();
            $xml = $domXml;
        }
    }

    private function removerTagsVazias(\DOMDocument $xml) {
        $xpath = new \DOMXPath($xml);
        foreach($xpath->query('//*[not(node())]') as $node) {
            if (!$node->hasAttributes()) {
                $node->parentNode->removeChild($node);
            }
        }
        return $xml;
    }

    private function limparListaRps() {
        $this->listaRps = array();
    }

    private function enviarLoteRpsWS($xml, $cdempresa, $inscricao, $numeroLote) {
        $nomeFuncao = "RecepcionarLoteRpsV3";
        
        $prefixoArquivo = $cdempresa.$inscricao.$numeroLote."_".$nomeFuncao."_";
        $nomeArquivoEnvio = $prefixoArquivo."env.xml";
        //file_put_contents($this->caminhoXml . $nomeArquivoEnvio, $xml);
        try {
            $resultado = $this->enviarParaWebservice($nomeFuncao, $xml);
        }
        catch (\Exception $ex) {
            throw new \Exception($ex->getMessage());
        }

        
        if ($resultado->outputXML) {
            $nomeArquivoResposta = $prefixoArquivo."resp.xml";
            //file_put_contents($this->caminhoXml . $nomeArquivoResposta, $resultado->outputXML);

            $domResult = new \DOMDocument("1.0", "UTF-8");
            $domResult->loadXML($resultado->outputXML);
            
            if ($domResult->getElementsByTagName("NumeroLote")->length > 0) {

                $protocolo = $domResult->getElementsByTagName("Protocolo")->item(0)->nodeValue;

                foreach ($this->listaAuxRps as $nota) {
                    Util::atualizarNotaTransmissao($this->connection, $nota["CDSERIERPS"], $nota["NRRPS"], $nota["CDEMPLOTERPS"], $nota["CDINSCESTA"], $nota["CDINSCMUNI"], $nota["NRSEQLOTERPS"], $nota["CDFILIAL"], $nota["NRLANCTONF"], $this->environment->getNrOrg(), $this->environment->getCdOperador());
                }
                
                Util::atualizarLote($this->connection, $this->listaAuxRps[0]["CDEMPLOTERPS"], $this->listaAuxRps[0]["CDINSCESTA"], $this->listaAuxRps[0]["CDINSCMUNI"], $this->NumeroLote, 2, $protocolo, $this->environment->getNrOrg(), $this->environment->getCdOperador());
                
                return true;
            }
            else if ($domResult->getElementsByTagName("MensagemRetorno")->length > 0) {
                $erros = array("TIPO" => 2);
                $erros["XML"] = $xml;
                
                foreach ($domResult->getElementsByTagName("MensagemRetorno") as $erro) {
                    
                    $erros[] = array(
                        "CODIGO" => Util::getChildNodeByName($erro, "Codigo")->nodeValue,
                        "DESCRICAO" => Util::getChildNodeByName($erro, "Mensagem")->nodeValue
                    );
                }
                return $erros;
            }
        }
        else {
            return false;
        }
    }

    private function consultarLoteRpsWS($xml, $cdempresa, $inscricao, $numeroLote) {
        $nomeFuncao = "ConsultarLoteRpsV3";
        $prefixoArquivo = $cdempresa.$inscricao.$numeroLote."_".$nomeFuncao."_";
        $nomeArquivoEnvio = $prefixoArquivo."env.xml";
        //file_put_contents($this->caminhoXml . $nomeArquivoEnvio, $xml);
        
        try {
            $resultado = $this->enviarParaWebservice($nomeFuncao, $xml);
        }
        catch (\Exception $ex) {
            throw new \Exception($ex->getMessage());
        }
        
        $nomeArquivoResposta = $prefixoArquivo."resp.xml";
        if ($resultado->outputXML) {
            //file_put_contents($this->caminhoXml . $nomeArquivoResposta, $resultado->outputXML);
            return $resultado->outputXML;
        }
        else {
            return false;
        }
    }
    
    private function consultarSituacaoLoteRpsWS($xml, $cdempresa, $inscricao, $numeroLote) {
        $nomeFuncao = "ConsultarSituacaoLoteRpsV3";
        $prefixoArquivo = $cdempresa.$inscricao.$numeroLote."_".$nomeFuncao."_";
        $nomeArquivoEnvio = $prefixoArquivo."env.xml";
        //file_put_contents($this->caminhoXml . $nomeArquivoEnvio, $xml);
        
        try {
            $resultado = $this->enviarParaWebservice($nomeFuncao, $xml);
        }
        catch (\Exception $ex) {
            throw new \Exception($ex->getMessage());
        }
        
        $nomeArquivoResposta = $prefixoArquivo."resp.xml";
        if ($resultado->outputXML) {
            //file_put_contents($this->caminhoXml . $nomeArquivoResposta, $resultado->outputXML);
            return $resultado->outputXML;
        }
        else {
            return false;
        }
    }

    public function consultarLoteRps($lote) {
        $this->cdempresa = $lote["CDEMPLOTERPS"];
        $this->cdinscmuni = str_replace(array(".","-","/"," "), "", $lote["CDINSCMUNI"]);
        $situacao = 0;
        $this->ambiente = Util::getAmbiente($this->connection, $lote["CDEMPLOTERPS"], $lote["CDINSCMUNI"], $this->environment->getNrOrg());
        if ($this->ambiente == null) $this->ambiente = 2;
    
        $this->senhaCertificadoDigital = Util::getSenhaCertificadoDigital($this->connection, $lote["CDEMPLOTERPS"], $lote["CDINSCMUNI"], $this->environment->getNrOrg());
        
        //$this->RazaoSocialPrestador = $lote["NMRAZSOCEMPR"];
        $inscricaoJuridica = str_replace(array("-", "/", ".", " "), "", $lote["NRINSJURIEST"]);
        $inscricaoMunicipal = str_replace(array("-", "/", ".", " "), "", $lote["CDINSCMUNI"]);
        
        $this->xmlConsultarSituacaoLoteRps = $this->gerarXmlConsultarSituacaoLoteRps($inscricaoJuridica, $inscricaoMunicipal, $lote["NRPROTOCOLORPS"]);
        $this->removeacento($this->xmlConsultarSituacaoLoteRps);
        
        $this->assinarXml(array("ConsultarSituacaoLoteRpsEnvio"), $this->xmlConsultarSituacaoLoteRps);

        $resultado = $this->consultarSituacaoLoteRpsWS($this->xmlConsultarSituacaoLoteRps, $lote["CDEMPLOTERPS"], $lote["NRINSJURIEST"], $lote["NRSEQLOTERPS"]);


        if (is_string($resultado)) {
            $domResult = new \DOMDocument("1.0", "UTF-8");
            $domResult->loadXML($resultado);

            $resposta = $domResult->getElementsByTagName("ConsultarSituacaoLoteRpsResposta");

            if($resposta->length > 0) {
                $situacao = Util::getChildNodeByName($resposta->item(0), "Situacao")->nodeValue;
   
                if($situacao == '2') {
                                             
                    $erros[] = array(
                        "CODIGO" => 'E92',
                        "DESCRICAO" => "Lote ainda não foi processado. Tente novamente mais tarde"
                    );
                    
                    return $erros;  
                }
            }
            else if ($domResult->getElementsByTagName("MensagemRetorno")->length > 0) {
                
                foreach ($domResult->getElementsByTagName("MensagemRetorno") as $erro) {
                    
                    $erros[] = array(
                        "CODIGO" => Util::getChildNodeByName($erro, "Codigo")->nodeValue,
                        "DESCRICAO" => Util::getChildNodeByName($erro, "Mensagem")->nodeValue
                    );
                }
                return $erros;
            }
            
        }

        if($situacao == '3' || $situacao == '4') {
            $xml = $this->gerarXmlConsultarLoteRps($inscricaoJuridica, $inscricaoMunicipal, $lote["NRPROTOCOLORPS"]);

            $this->assinarXml(array("ConsultarLoteRpsEnvio"), $this->xmlConsultarLoteRps);

            $resultado = $this->consultarLoteRpsWS($this->xmlConsultarLoteRps, $lote["CDEMPLOTERPS"], $lote["NRINSJURIEST"], $lote["NRSEQLOTERPS"]);
     
            if (is_string($resultado)) {
                $domResult = new \DOMDocument("1.0", "UTF-8");
                $domResult->loadXML($resultado);
                
                if ($domResult->getElementsByTagName("ListaNfse")->length > 0) {
    
                    $nodeList = $domResult->getElementsByTagName("InfNfse");
                    
                    if ($nodeList->length > 0) {
                        foreach ($nodeList as $nota) {
                            $numeroNota = Util::getChildNodeByName($nota, "Numero")->nodeValue;
                            $codigoVerificacao = Util::getChildNodeByName($nota, "CodigoVerificacao")->nodeValue;
                            $nodeIdentificacaoRps = $nota->getElementsByTagName("IdentificacaoRps")->item(0);
                            $serieRps = Util::getChildNodeByName($nodeIdentificacaoRps, "Serie")->nodeValue;
                            $numeroRps = Util::getChildNodeByName($nodeIdentificacaoRps, "Numero")->nodeValue;
                            $numeroRps = str_pad($numeroRps, 15, "0", STR_PAD_LEFT);
                            
                            /*==================================*/
                            $dataEmissao = Util::getChildNodeByName($nota, "DataEmissao")->nodeValue;
                            $dataEmissao = date_create_from_format("Y-m-d\TH:i:s", $dataEmissao);
                            $dataEmissao = $dataEmissao->format("d/m/Y");
                            
                            $competencia = $nota->getElementsByTagName("Competencia")->item(0)->nodeValue;
                            $competencia = date_create_from_format("Y-m-d", $competencia);
                            $competencia = $competencia->format("d/m/Y");
                            /*==================================*/
        
                            Util::atualizarNumeroRps($this->connection, $serieRps, $numeroRps, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], $numeroNota, $codigoVerificacao, 4, $dataEmissao, $competencia, $this->environment->getNrOrg(), $this->environment->getCdOperador());
        
                            Util::atualizarNotaConsulta($this->connection, $serieRps, $numeroRps, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], $numeroNota, $this->environment->getNrOrg(), $this->environment->getCdOperador());
                            
                            Util::inserirArquivoXml($this->connection, $serieRps, $numeroRps, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], "A", $resultado, $this->environment->getNrOrg(), $this->environment->getCdOperador());
                        }
                        
                        Util::atualizarLote($this->connection, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], 4, $lote["NRPROTOCOLORPS"], $this->environment->getNrOrg(), $this->environment->getCdOperador());
                        
                        return true;
                    }
                    else {
                        return null;
                    }
                }
                else {
                    $erros = array();
                    $nodeList = $domResult->getElementsByTagName("MensagemRetorno");

                    foreach ($nodeList as $erro) {
                        $codigoErro = Util::getChildNodeByName($erro, "Codigo")->nodeValue;
                        $descricaoErro = Util::getChildNodeByName($erro, "Mensagem")->nodeValue;
                        
                        $erros[] = array(
                            "CODIGO" => $codigoErro,
                            "DESCRICAO" => $descricaoErro
                        );
                        
                        // E92 = lote ainda nao foi processado
                        if ($codigoErro == "E92" || $codigoErro == "92") {
                            return $erros;
                        }
                    }
                    
                    /*$nota = $domResult->getElementsByTagName("MensagemRetorno")[0];
                    $nodeIdentificacaoRps = $nota->getElementsByTagName("IdentificacaoRps")->item(0);
                    $serieRps = Util::getChildNodeByName($nodeIdentificacaoRps, "Serie")->nodeValue;*/
                    
                    Util::atualizarSituacaoNumeroRpsPeloLote($this->connection, null, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], 3, $this->environment->getNrOrg(), $this->environment->getCdOperador());
                    
                    Util::atualizarLote($this->connection, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], 3, $lote["NRPROTOCOLORPS"], $this->environment->getNrOrg(), $this->environment->getCdOperador());
                    
                    return $erros;
                }
            }
            else {
                return false;
            }
        }
        else {
            $erros[] = array(
                "CODIGO" => '2',
                "DESCRICAO" => 'Não foi possivel consultar o lote. Tente novamente mais tarde.'
            );
        }

    }
    
    private function gerarXmlCancelarNota($nrnotafisc, $inscricaoJuridica, $inscricaoMunicipal, $codMotivoCancelamento, $cdmunicibge) {
        $this->xmlCancelarNota =
        '<ns:CancelarNfseEnvio xmlns:ns="http://www.ginfes.com.br/servico_cancelar_nfse_envio_v03.xsd" xmlns:tipos="http://www.ginfes.com.br/tipos_v03.xsd">
            <Pedido>
                <tipos:InfPedidoCancelamento
                    Id="'.$nrnotafisc.'">
                    <tipos:IdentificacaoNfse>
                        <tipos:Numero>'.$nrnotafisc.'</tipos:Numero>
                        <tipos:Cnpj>'.$inscricaoJuridica.'</tipos:Cnpj>
                        <tipos:InscricaoMunicipal>'.$inscricaoMunicipal.'</tipos:InscricaoMunicipal>
                        <tipos:CodigoMunicipio>'.$cdmunicibge.'</tipos:CodigoMunicipio>
                    </tipos:IdentificacaoNfse>
                    <tipos:CodigoCancelamento>'.str_pad($codMotivoCancelamento, 4, "0", STR_PAD_LEFT).'</tipos:CodigoCancelamento>
                </tipos:InfPedidoCancelamento>
            </Pedido>
        </ns:CancelarNfseEnvio>';
    }
    
    public function cancelarNotas($notas, $codMotivo, $dsMotivo, $cdmunicibge, $cdmunicsiafi) {
        $this->cdempresa = $notas[0]["CDEMPRESA"];
        $this->cdinscmuni = str_replace(array(".","-","/"," "), "", $notas[0]["CDINSCMUNI"]);
        
        $this->ambiente = Util::getAmbiente($this->connection, $notas[0]["CDEMPRESA"], $notas[0]["CDINSCMUNI"], $this->environment->getNrOrg());
        if ($this->ambiente == null) $this->ambiente = 2;

        $this->senhaCertificadoDigital = Util::getSenhaCertificadoDigital($this->connection, $notas[0]["CDEMPRESA"], $notas[0]["CDINSCMUNI"], $this->environment->getNrOrg());
        
        //$this->RazaoSocialPrestador = $notas[0]["NMRAZSOCEMPR"];
        $errosTotais = array();
        
        foreach ($notas as $nota) {
            $inscricaoJuridica = str_replace(array("-", "/", ".", " "), "", $nota["NRINSJURIEST"]);
            $inscricaoMunicipal = str_replace(array("-", "/", ".", " "), "", $nota["CDINSCMUNI"]);
            $this->gerarXmlCancelarNota($nota["NRNOTAFISC"], $inscricaoJuridica, $inscricaoMunicipal, $codMotivo, $cdmunicibge);
            $this->assinarXml(['CancelarNfseEnvio'], $this->xmlCancelarNota);
            
            $resultado = $this->cancelarNotaWS($this->xmlCancelarNota, $nota["CDEMPRESA"], $nota["NRINSJURIEST"], $nota["NRNOTAFISC"]);
    
            if (is_string($resultado)) {
                $domResult = new \DOMDocument("1.0", "UTF-8");
                $domResult->loadXML($resultado);
                $nodeList = $domResult->getElementsByTagName("MensagemRetorno");
                if ($nodeList->length > 0) {
                    $errosNota = array();
                    foreach ($nodeList as $erro) {
                        $errosNota[] = array(
                            "CODIGO" => Util::getChildNodeByName($erro, "Codigo")->nodeValue,
                            "DESCRICAO" => Util::getChildNodeByName($erro, "Mensagem")->nodeValue
                        );
                    }
                    $errosTotais[$nota["NRLANCTONF"]] = $errosNota;
                }
                else {
                   $motivos = array(
                        1 => "Erro na emissão",
                        2 => "Serviço não prestado",
                        3 => "Erro na assinatura", 
                        4 => "Duplicidade da nota",
                        5 => "Erro no Processamento"
                    );
                    
                    $descMotivo = strtr($codMotivo, $motivos);
    
                    Util::inserirNotaCancelada($this->connection, "S", "C", $descMotivo, "C", $nota["CDFILIAL"], $nota["NRLANCTONF"], $this->environment->getNrOrg(), $this->environment->getCdOperador(), $this->newCode);
                    
                    Util::atualizarNotaCancelada($this->connection, "N", $nota["CDFILIAL"], $nota["NRLANCTONF"], $this->environment->getNrOrg());
                }
            }
            else {
                return false;
            }
        }
        return (count($errosTotais) > 0) ? $errosTotais : true;
    }
    
    private function cancelarNotaWS($xml, $cdempresa, $inscricao, $nrnotafisc) {
        $nomeFuncao = "CancelarNfseV3";
        $prefixoArquivo = $cdempresa.$inscricao.$nrnotafisc."_".$nomeFuncao."_";
        $nomeArquivoEnvio = $prefixoArquivo."env.xml";
        //file_put_contents($this->caminhoXml . $nomeArquivoEnvio, $xml);
        
        try {
            $resultado = $this->enviarParaWebservice($nomeFuncao, $xml);
        }
        catch (\Exception $ex) {
            //file_put_contents($this->caminhoXml . $prefixoArquivo."resp-error.xml", $ex->getMessage());
            throw new \Exception($ex->getMessage());
        }
        
        $nomeArquivoResposta = $prefixoArquivo."resp.xml";
        
        if ($resultado->outputXML) {
            //file_put_contents($this->caminhoXml . $nomeArquivoResposta, $resultado->outputXML);
            return $resultado->outputXML;
        }
        else {
            return false;
        }
    }
    
    private function enviarParaWebservice($nomeFuncao, $conteudo) {
        if($this->caminhoCertificado) {
            $caminho = $this->caminhoCertificado . $this->cdempresa . "_" . $this->cdinscmuni . DIRECTORY_SEPARATOR . "certKey.pem";
            $unlink  = false;
        }
        else {
            $certKey = $this->linkCertKey;
            $hash    = uniqid();
            if (is_dir("/tmp")) {
                $caminho= "/tmp/certKey$hash.pem";
            }
            else {
                $caminho = __DIR__ . DIRECTORY_SEPARATOR ."certKey$hash.pem";
            }
            file_put_contents($caminho, file_get_contents($certKey));
            $unlink = true;
        }

        $cabecalho = '&lt;?xml version="1.0" encoding="UTF-8"?&gt;&lt;ns:cabecalho versao="3" xmlns:ns="http://www.ginfes.com.br/cabecalho_v03.xsd"&gt;&lt;versaoDados&gt;3&lt;/versaoDados&gt;&lt;/ns:cabecalho&gt;';

        $conteudo = str_replace("<", "&lt;", $conteudo);
        $conteudo = str_replace(">", "&gt;", $conteudo);
        $conteudo = $this->createSoapText($conteudo, $nomeFuncao, $cabecalho);
        $soapVar = new \SoapVar($conteudo, XSD_ANYXML);
        $lotUri = ($this->ambiente == 1) ? self::URL_PRODUCAO : self::URL_HOMOLOGACAO;
        $resultado = $this->buildSoapCall(
            $caminho,
            $lotUri,
            $lotUri,
            $nomeFuncao,
            $soapVar,
            $conteudo
        );

        $resultado->outputXML = str_replace("&quot;", '"', $resultado->outputXML);
        $resultado->outputXML = str_replace("ns2:", '', $resultado->outputXML);

        if($unlink) unlink($caminho);
        return $resultado;
    }
    
    protected function assinaXml(&$xml, $tags) {
        $local = ($this->caminhoCertificado);
        if($local) {
            $dir = $this->caminhoCertificado . $this->cdempresa . "_" . $this->cdinscmuni;
            if(!is_dir($dir)) return false;
            if((!is_file($dir . DIRECTORY_SEPARATOR . "certificado.pfx")) && (!is_file($dir . DIRECTORY_SEPARATOR . "certificado.p12"))) return false;
            else {
                if(is_file($dir . DIRECTORY_SEPARATOR . "certificado.pfx")) $data = file_get_contents($dir . DIRECTORY_SEPARATOR . "certificado.pfx");
                else if(is_file($dir . DIRECTORY_SEPARATOR . "certificado.p12")) $data = file_get_contents($dir . DIRECTORY_SEPARATOR . "certificado.p12");
            }
        }
        else {
            try{
                $data = file_get_contents($this->linkCertificado);
            }
            catch(\Exception $e) {
                return false;
            }
            if(empty($data)) return false;
        }
        
        $certificado = Certificate::readPfx($data, $this->senhaCertificadoDigital);
        foreach ($tags as $tag) {
            $xml = Signer::sign($certificado, $xml, $tag, "Id", $algorithm = OPENSSL_ALGO_SHA1, [false, false, null, null]);
        }
   
        return $xml;
        //var_dump($certificado);die;
    }
    
    public function createSoapText($conteudo, $nomeFuncao, $cabecalho) {
        if($this->ambiente == 2) {
             $conteudo = "<soap:Envelope xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\"><soap:Body><$nomeFuncao xmlns=\"http://homologacao.ginfes.com.br\"><arg0 xmlns=\"\">$cabecalho</arg0><arg1 xmlns=\"\">$conteudo</arg1></$nomeFuncao></soap:Body></soap:Envelope>";
        }
        else {
            $conteudo = "<soap:Envelope xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\"><soap:Body><$nomeFuncao xmlns=\"http://producao.ginfes.com.br\"><arg0 xmlns=\"\">$cabecalho</arg0><arg1 xmlns=\"\">$conteudo</arg1></$nomeFuncao></soap:Body></soap:Envelope>";
        }
        
        return $conteudo;
        
    }
    
    public function buildSoapCall($cert, $location, $wsdl, $functionName, $soapVar, $xmlLote) {
       // var_dump($cert, $location, $wsdl, $functionName, $soapVar, $xmlLote);die;
       $functionName.= 'Request';
        try {
            $msgSize = strlen($xmlLote);
            $parameters = [
                "Content-Type: text/xml;charset=UTF-8",
                "SOAPAction: \"http://homologacao.ginfes.com.br/ServiceGinfesImpl/$functionName\"",
                "Content-length: $msgSize"
            ];

            $oCurl = curl_init();
            curl_setopt($oCurl, CURLOPT_URL, $wsdl);
            curl_setopt($oCurl, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4);
            curl_setopt($oCurl, CURLOPT_HEADER, 1);
            curl_setopt($oCurl, CURLOPT_SSL_VERIFYHOST, 0);
            curl_setopt($oCurl, CURLOPT_SSL_VERIFYPEER, 0);
            curl_setopt($oCurl, CURLOPT_SSLVERSION, 0);
            curl_setopt($oCurl, CURLOPT_SSLCERT, $cert);
            curl_setopt($oCurl, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($oCurl, CURLOPT_POST, true);
            curl_setopt($oCurl, CURLOPT_POSTFIELDS, $xmlLote);
            curl_setopt($oCurl, CURLOPT_HTTPHEADER, $parameters);
            
            $response = curl_exec($oCurl);
            $this->soaperror = curl_error($oCurl);
            $ainfo = curl_getinfo($oCurl);
            if (is_array($ainfo)) {
                $this->soapinfo = $ainfo;
            }
            $headsize = curl_getinfo($oCurl, CURLINFO_HEADER_SIZE);
            $httpcode = curl_getinfo($oCurl, CURLINFO_HTTP_CODE);
            curl_close($oCurl);
            $responseHead = trim(substr($response, 0, $headsize));
            $responseBody = trim(substr($response, $headsize));
            if ($httpcode == 404) {
                throw new \Exception('Erro 404.', 1);
            } else if ($httpcode == 500) {
                throw new \Exception($responseBody, 1);
            } else if ($httpcode != 200) {
                throw new \Exception('Erro inesperado.', 1);
            } else {

                $inicioResponse = strpos($responseBody, "&lt;");
                $length         = strrpos($responseBody, "&gt;") - $inicioResponse + 4;
                if($inicioResponse != strlen($responseBody)) {
                    $responseBody   = substr($responseBody, $inicioResponse, $length);
                }
                
                $responseBody   = str_replace("&lt;", "<", $responseBody);
                $responseBody   = str_replace("&gt;", ">", $responseBody);
                $responseBody   = str_replace("&quot;", '"', $responseBody);
                $responseBody   = str_replace(array("ns1:", "ns2:", "ns3:", "ns4:", "ns5:"), '', $responseBody);
                $response       = new \stdClass();
                $response->outputXML = $responseBody;
                return $response;
            }

        } catch(\Exception $e) {
            throw $e;
        }
    }

/*    private function assinaXml(\DOMDocument $domXml, array $tags, $id = '') {
	    $local = ($this->caminhoCertificado);
        if($local) {
            $dir = $this->caminhoCertificado . $this->cdempresa . "_" . $this->cdinscmuni;
            if(!is_dir($dir)) return false;
            if((!is_file($dir . DIRECTORY_SEPARATOR . "certificado.pfx")) && (!is_file($dir . DIRECTORY_SEPARATOR . "certificado.p12"))) return false;
            else {
                if(is_file($dir . DIRECTORY_SEPARATOR . "certificado.pfx")) $data = file_get_contents($dir . DIRECTORY_SEPARATOR . "certificado.pfx");
                else if(is_file($dir . DIRECTORY_SEPARATOR . "certificado.p12")) $data = file_get_contents($dir . DIRECTORY_SEPARATOR . "certificado.p12");
            }
        }
        else {
            try{
                $data = file_get_contents($this->linkCertificado);
            }
            catch(\Exception $e) {
                return false;
            }
            if(empty($data)) return false;
        }
        
        openssl_pkcs12_read($data, $cert, $this->senhaCertificadoDigital);

        if (is_null($cert)) {
            return false;
        }
        else {
            //file_put_contents($dir.DIRECTORY_SEPARATOR."certKey.pem", $cert["pkey"]."\r\n".$cert["cert"]);
            //file_put_contents($dir.DIRECTORY_SEPARATOR."priKey.pem", $cert["pkey"]);
            //file_put_contents($dir.DIRECTORY_SEPARATOR."pubKey.pem", $cert["cert"]);

            $this->publicKey = $this->getPublicKey($cert["cert"]);
            $this->privateKeyId = openssl_get_privatekey($cert["pkey"]);

            foreach ($tags as $tag) {
                $this->assinaXmlAux($domXml, $tag, $id);
            }

            openssl_free_key($this->privateKeyId);
            return $domXml;
        }

    }*/
    
    private function assinaXmlAux(\DOMDocument $domXml, $tag, $id = '') {
		$nodes = $domXml->getElementsByTagName($tag);
		foreach ($nodes as $node) {
			$root = $node->parentNode;

			$id = ($id) ? $id : trim($node->getAttribute("Id"));
            
			$dados = $node->C14N(false, false, null, null);
			$hashValue = hash("sha1", $dados, true);
			$digValue = base64_encode($hashValue);

            $signature = $domXml->createElement("Signature");
            $root->appendChild($signature);
            $signature->setAttribute("xmlns", "http://www.w3.org/2000/09/xmldsig#");
            if($id) {
                $signature->setAttribute("Id", "Ass_".$id);
            }

            $signedInfo = $domXml->createElement("SignedInfo");
	        $signature->appendChild($signedInfo);

	        $novaTag = $domXml->createElement("CanonicalizationMethod");
	        $signedInfo->appendChild($novaTag);
	        $novaTag->setAttribute("Algorithm", "http://www.w3.org/TR/2001/REC-xml-c14n-20010315");

	        $novaTag = $domXml->createElement("SignatureMethod");
	        $signedInfo->appendChild($novaTag);
	        $novaTag->setAttribute("Algorithm", "http://www.w3.org/2000/09/xmldsig#rsa-sha1");

	        $reference = $domXml->createElement("Reference");
	        $signedInfo->appendChild($reference);
	        if($id) {
	            $reference->setAttribute("URI", "#".$id);
	        }
	        else {
	           $reference->setAttribute("URI", ""); 
	        }

	        $transforms = $domXml->createElement("Transforms");
	        $reference->appendChild($transforms);

	        $novaTag = $domXml->createElement("Transform");
	        $transforms->appendChild($novaTag);
	        $novaTag->setAttribute("Algorithm", "http://www.w3.org/2000/09/xmldsig#enveloped-signature");

	        $novaTag = $domXml->createElement("Transform");
	        $transforms->appendChild($novaTag);
	        $novaTag->setAttribute("Algorithm", "http://www.w3.org/TR/2001/REC-xml-c14n-20010315");

	        $novaTag = $domXml->createElement("DigestMethod");
	        $reference->appendChild($novaTag);
	        $novaTag->setAttribute("Algorithm", "http://www.w3.org/2000/09/xmldsig#sha1");

	        $novaTag = $domXml->createElement("DigestValue", $digValue);
	        $reference->appendChild($novaTag);

            $dados = $signedInfo->C14N(false, false, null, null);
            $assinatura = "";
            openssl_sign($dados, $assinatura, $this->privateKeyId);
            $signatureValue = base64_encode($assinatura);


	        $novaTag = $domXml->createElement("SignatureValue", $signatureValue);
	        $signature->appendChild($novaTag);

	        $keyInfo = $domXml->createElement("KeyInfo");
	        $signature->appendChild($keyInfo);

	        $X509Data = $domXml->createElement("X509Data");
	        $keyInfo->appendChild($X509Data);

	        $novaTag = $domXml->createElement("X509Certificate", $this->publicKey);
	        $X509Data->appendChild($novaTag);
    	}
        return $domXml;
	}

    private function getPublicKey($key)
    {
        $data = "";
        $arCert = explode("\n", $key);
        foreach ($arCert as $curData) {
            //remove a tag de inicio e fim do certificado
            if (strncmp($curData, "-----BEGIN CERTIFICATE", 22) != 0 &&
                    strncmp($curData, "-----END CERTIFICATE", 20) != 0 ) {
                //carrega o resultado numa string
                $data .= trim($curData);
            }
        }
        return $data;
    }

    private function gerarXmlConsultarLoteRps($inscricaoPrestador, $inscricaoMunicipalPrestador, $protocolo) {
        $this->xmlConsultarLoteRps = '<ConsultarLoteRpsEnvio xmlns="http://www.ginfes.com.br/servico_consultar_lote_rps_envio_v03.xsd" xmlns:tipos="http://www.ginfes.com.br/tipos_v03.xsd" Id="'.$protocolo.'"><Prestador><tipos:Cnpj>'.$inscricaoPrestador.'</tipos:Cnpj><tipos:InscricaoMunicipal>'.$inscricaoMunicipalPrestador.'</tipos:InscricaoMunicipal></Prestador><Protocolo>'.$protocolo.'</Protocolo></ConsultarLoteRpsEnvio>';

        return $this->xmlConsultarLoteRps;
    }
    
    private function gerarXmlConsultarSituacaoLoteRps($inscricaoPrestador, $inscricaoMunicipalPrestador, $protocolo) {
        $this->xmlConsultarSituacaoLoteRps = '<ConsultarSituacaoLoteRpsEnvio xmlns="http://www.ginfes.com.br/servico_consultar_situacao_lote_rps_envio_v03.xsd" xmlns:tipos="http://www.ginfes.com.br/tipos_v03.xsd" Id="'.$protocolo.'"><Prestador><tipos:Cnpj>'.$inscricaoPrestador.'</tipos:Cnpj><tipos:InscricaoMunicipal>'.$inscricaoMunicipalPrestador.'</tipos:InscricaoMunicipal></Prestador><Protocolo>'.$protocolo.'</Protocolo></ConsultarSituacaoLoteRpsEnvio>';

        return $this->xmlConsultarSituacaoLoteRps;
    }
    
    public function enviarLoteRps($notas) {
        $erros = array("TIPO" => 1);
        $this->listaAuxRps = array();
        
        $nrlanctonf = $notas[0]["NRLANCTONF"];
        $cdfilial = $notas[0]["CDFILIAL"];
        
        $notafiscal = Util::getCorpoNF($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());
        
        $this->cdempresa = $notafiscal["CDEMPRESA"];
        $this->cdinscmuni = str_replace(array(".","-","/"," "), "", $notafiscal["CDINSCMUNIEMP"]);
        
        $this->ambiente = Util::getAmbiente($this->connection, $notafiscal["CDEMPRESA"], $notafiscal["CDINSCMUNIEMP"], $this->environment->getNrOrg());
        if ($this->ambiente == null) $this->ambiente = 2;

        $this->senhaCertificadoDigital = Util::getSenhaCertificadoDigital($this->connection, $notafiscal["CDEMPRESA"], $notafiscal["CDINSCMUNIEMP"], $this->environment->getNrOrg());

        $this->NumeroLote = Util::inserirLoteRps($this->connection, $notafiscal["CDEMPRESA"], $notafiscal["CDINSCESTA"], $notafiscal["CDINSCMUNIEMP"], $notafiscal["NRINSJURIEST"], $this->environment->getNrOrg(), $this->environment->getCdOperador(), $this->newCode);
        $this->IdLote = $this->NumeroLote;
        $this->InscricaoPrestador = str_replace(array(".","-","/"), "", $notafiscal["NRINSJURIEST"]);
        $this->InscricaoMunicipalPrestador = str_replace(array(".","-","/"), "", $notafiscal["CDINSCMUNIEMP"]);
        $this->RazaoSocialPrestador = $notafiscal["NMRAZSOCEMIT"];
        
        $this->CodigoMunicipioPrestador = $notafiscal["CDMUNICIBGEEMP"];

        foreach ($notas as $nota) {
            $nrlanctonf = $nota["NRLANCTONF"];
            $cdfilial = $nota["CDFILIAL"];
            $notafiscal = Util::getCorpoNF($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());

            //VALIDACAO
            if (empty($notafiscal["CDINSCMUNIEMP"])) {
                $erros[] = array(
                    "CODIGO" => $notafiscal["NRLANCTONF"],
                    "DESCRICAO" => "Inscrição municipal deve ser relacionada com a empresa informada."
                );
            }
            //VALIDACAO
            if ($notafiscal["IDNATUOPEREMPR"] == "99") {
                $erros[] = array(
                    "CODIGO" => $notafiscal["NRLANCTONF"],
                    "DESCRICAO" => "Natureza da operação deve ser informada."
                );
            }
            //VALIDACAO
            if ($notafiscal["IDREGESPTRIBEMP"] == "99") {
                $erros[] = array(
                    "CODIGO" => $notafiscal["NRLANCTONF"],
                    "DESCRICAO" => "Regime especial de tributação deve ser informado."
                );
            }

            $this->NaturezaOperacao = (int)$notafiscal["IDNATUOPEREMPR"];
            $this->RegimeEspecialTributacao = (int) $notafiscal["IDREGESPTRIBEMP"];
            $this->OptanteSimplesNacional = 2;

            if ($notafiscal["IDOPTSIMPLES"] == "S") {
                $this->RegimeEspecialTributacao = (int)$notafiscal["IDREGESPTRIBEMP"];
                $this->OptanteSimplesNacional = 1;
            }
            $this->IncentivadorCultural = 2;
            
            $dataAtual = date_create();
            //$this->DataEmissao = $dataAtual->format("Y-m-d\TH:i:s");
            $this->DataEmissao = $notafiscal["DTENTRSAID"] . "T" . $dataAtual->format("H:i:s");

            $this->NumeroRps = Util::inserirNumeroRps($this->connection, $this->NumeroLote, 2, $notafiscal["CDEMPRESA"], $notafiscal["CDSERIERPS"], null, $notafiscal["CDINSCESTA"], $notafiscal["CDINSCMUNIEMP"], $notafiscal["NRINSJURIEST"], null, null, $this->environment->getNrOrg(), $this->environment->getCdOperador(), $this->newCode);

            $this->SerieRps = $notafiscal["CDSERIERPS"];
            $this->IdRps = $this->NumeroRps;
            $this->TipoRps = 1;
            $this->StatusRps = 1;

            $this->InscricaoTomador = str_replace(array(".","-","/"," "), "", $notafiscal["NRINSJURCLIE"]);
            $this->InscricaoMunicipalTomador = str_replace(array(".","-","/"," "), "", $notafiscal["NRINSMUNICLIE"]);
            if (strtoupper($this->InscricaoMunicipalTomador) == "ISENTO" || strtoupper($this->InscricaoMunicipalTomador) == "ISENTA") {
                $this->InscricaoMunicipalTomador = "";
            }
            $this->RazaoSocialTomador = $notafiscal["NMRAZSOCCLIE"];
            $this->Endereco = substr($notafiscal["DSENDECLIE"], 0, 125);
            $this->Numero = ".";
            $this->Complemento = substr($notafiscal["DSCOMPENDCLI"], 0, 60);
            $this->Bairro = substr($notafiscal["NMBAIRCLIE"], 0, 60);
            $this->CodigoMunicipioTomador = $notafiscal["CDMUNICIBGECLIE"];
            $this->Uf = $notafiscal["SGESTADOCLIE"];
            $this->Cep = substr(str_replace(array(".","-"), "", $notafiscal["NRCEPCLIE"]), 0, 8);
            $this->Telefone = substr(str_replace(array("(", ")", "-", " "), "", $notafiscal["NRTELECLIE"]), 0, 11);
            $this->Email = substr($notafiscal["DSEMAILCLIENFE"], 0, 80);

            $this->ValorServicos = $notafiscal["VRPRODUTOS"];
            $valorLiquido = $notafiscal["VRPRODUTOS"];
            $this->ValorDeducoes = 0.0;
            $this->ValorPis = 0.0;
            $this->ValorCofins = 0.0;
            $this->ValorInss = 0.0;
            $this->ValorIr = 0.0;
            $this->ValorCsll = 0.0;
            $this->IssRetido = 2;
            $this->ValorIss = 0.0;
            $this->ValorIssRetido = 0.0;
            $this->OutrasRetencoes = 0.0;
            $this->DescontoCondicionado = 0.0;
            $this->DescontoIncondicionado = 0.0;
            $this->BaseCalculo = 0.0;

            $impostos = Util::getImpostos($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());
    
            foreach ($impostos as $imp) {
                if (in_array($imp["CDIMPOSTONFE"], array("50", "55", "56")) && $imp["IDTRATIMPO"] == "R") {
                    $this->ValorPis = (float)$imp["VALORIMPOSTO"];
                    $valorLiquido -= $this->ValorPis;
                }
                else if (in_array($imp["CDIMPOSTONFE"], array("60", "65", "66")) && $imp["IDTRATIMPO"] == "R") {
                    $this->ValorCofins = (float)$imp["VALORIMPOSTO"];
                    $valorLiquido -= $this->ValorCofins;
                }
                else if ($imp["CDIMPOSTONFE"] == "42" && $imp["IDTRATIMPO"] == "R") {
                    $this->ValorInss = (float)$imp["VALORIMPOSTO"];
                    $valorLiquido -= $this->ValorInss;
                }
                else if (in_array($imp["CDIMPOSTONFE"], array("22", "25")) && $imp["IDTRATIMPO"] == "R") {
                    $this->ValorIr = (float)$imp["VALORIMPOSTO"];
                    $valorLiquido -= $this->ValorIr;
                }
                else if (in_array($imp["CDIMPOSTONFE"], array("70", "75", "76")) && $imp["IDTRATIMPO"] == "R") {
                    $this->ValorCsll = (float)$imp["VALORIMPOSTO"];
                    $valorLiquido -= $this->ValorCsll;
                }
                else if (in_array($imp["CDIMPOSTONFE"], array("30", "31", "35", "36"))) {
                    if ($imp["IDTRATIMPO"] == "R") {
                        $this->IssRetido = 1;
                        $this->ValorIssRetido = (float)$imp["VALORIMPOSTO"];
                        $valorLiquido -= $this->ValorIssRetido;
                        $this->BaseCalculo = $imp["BASECALCULO"];
                    }
                    else {
                        $this->ValorIss = (float)$imp["VALORIMPOSTO"];
                        $this->BaseCalculo = $imp["BASECALCULO"];
                    }
                }
            }

            $outrasRetencoes = Util::getOutrasRetencoes($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());
            if ($outrasRetencoes["VRIMPOSTOIT"] > 0) {
                $this->OutrasRetencoes = $outrasRetencoes["VALORIMPOSTO"];
                $valorLiquido -= $this->OutrasRetencoes;
            }

            $this->ValorLiquidoNfse = $valorLiquido;

            if (count($impostos) > 0) {
                $impostoItem = Util::getImpostoItemMaiorAliquota($this->connection, $nrlanctonf, $cdfilial, "31", $this->environment->getNrOrg());
                $this->Aliquota = 0;
                if (!empty($impostoItem)) {
                    $this->Aliquota = (float)$impostoItem["VRALIQUOTA"];
                }
            }
            else {
                //VALIDACAO
                $erros[] = array(
                    "CODIGO" => $nota["NRLANCTONF"],
                    "DESCRICAO" => "Impostos da nota devem ser informados."
                );
            }

            $itemProduto = Util::getItemProduto($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());
            $cdarvprod = $itemProduto[0]["CDARVPROD"];

            $descricaoProd = "";

            foreach ($itemProduto as $linha) {
                $descricaoProd .= $linha["DSPRODEMIS"] . "\r\n";
            }

            $itemEncargo = Util::getItemEncargo($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());
            //$descricaoEncargo = "";
            foreach ($itemEncargo as $item) {
                //$descricaoEncargo .= $item["DESCRICAO"] . "\n";
                if ($item["IDENCARGO"] == "D") {
                    $this->DescontoIncondicionado += (float)$item["VRITEMENCA"];
                }
            }

            /*$descricaoDadosAdic = "";
            $dadosAdicionais = Util::getDadosAdicionais($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());
            foreach ($dadosAdicionais as $item) {
                $descricaoDadosAdic .= $item["DSDADOADIC"];
            }

            $this->Discriminacao = $descricaoProd;
            if (!empty($descricaoEncargo)) {
                $this->Discriminacao .= "|" . $descricaoEncargo;
            }
            if (!empty($descricaoDadosAdic)) {
                $this->Discriminacao .= "|" . $descricaoDadosAdic;
            }*/

            $this->Discriminacao = trim($descricaoProd);
            //$this->Discriminacao = substr(Util::retiraAcento($this->Discriminacao), 0, 2000);
            $this->Discriminacao = substr($this->Discriminacao, 0, 2000);
            
            //$this->CodigoCnae = str_replace(array("-", "/", ".", " "), "", $notafiscal["CDCNAE"]);
            $this->CodigoCnae = "";

            $codTribMuni = Util::getCodTribMuni($this->connection, $notafiscal["CDEMPRESA"], $notafiscal["CDINSCMUNIEMP"], $cdarvprod, $this->environment->getNrOrg());
            
            //VALIDACAO
            if (!is_array($codTribMuni) || empty($codTribMuni["CDCLASSERVPADR"])) {
                $erros[] = array(
                    "CODIGO" => $notafiscal["NRLANCTONF"],
                    "DESCRICAO" => "Código da classificação do serviço deve ser informado."
                );
            }
            else {
                $this->ItemListaServico = $codTribMuni["CDCLASSERVPADR"];
                $this->CodigoTributacaoMunicipio = $codTribMuni["CDTRIBPRODPADR"];
            }
            
            $this->adicionarRpsEnviarLote($cdfilial, $nrlanctonf);
            
            $this->listaAuxRps[] = array(
                "CDSERIERPS"   => $notafiscal["CDSERIERPS"],
                "NRRPS"        => $this->NumeroRps,
                "CDEMPLOTERPS" => $notafiscal["CDEMPRESA"],
                "CDINSCESTA"   => $notafiscal["CDINSCESTA"],
                "CDINSCMUNI"   => $notafiscal["CDINSCMUNIEMP"],
                "NRSEQLOTERPS" => $this->NumeroLote,
                "NRINSJURIEST" => $notafiscal["NRINSJURIEST"],
                "CDFILIAL"     => $cdfilial,
                "NRLANCTONF"   => $nrlanctonf
            );
        }

        if (count($erros) > 1) {
            $this->limparListaRps();
            return $erros;
        }
        else {
            $this->gerarXmlEnviarLoteRps();
            $this->removeacento($this->xmlEnviarLoteRps);
            $this->assinarXml(array("LoteRps"), $this->xmlEnviarLoteRps);
            return $this->enviarLoteRpsWS($this->xmlEnviarLoteRps, $notafiscal["CDEMPRESA"], $notafiscal["NRINSJURIEST"], $this->NumeroLote);
        }
    }

   public function removeacento(&$text) {
       $from = 'ÀÁÃÂÉÊÍÓÕÔÚÜÇàáãâéêíóõôúüç ';
       $to   = 'aaaaeeiooouucaaaaeeiooouuc-';
       return strtr($text, $from, $to);
   }
 

}