<?php

namespace Teknisa\DF\Modules\Saida\Services;


use Teknisa\DF\Modules\Saida\Factories\Dfs65001CertificadoDigitalFactory;
use Zeedhi\Framework\Remote\FileUploader\Server;
use Teknisa\DF\Modules\Saida\NFSe\Utilities\Util;
use Zeedhi\Framework\Remote\cURLRequest;
use Zeedhi\Framework\HTTP\UploadedFile;
use Teknisa\Libs\Util\Utilities;

class Dfs65001CertificadoDigitalService {
    
    const DEV_LINK  = 'http://midiatest.teknisa.com/files/';
    const PROD_LINK = 'https://midia.teknisa.com/files/';
    const EXTENSION_PFX = 'pfx';
    const EXTENSION_P12 = 'p12';
    
    protected $fileUploader;
    protected $connection;
    protected $certificadoDigitalFactory;
    
    public function __construct(Dfs65001CertificadoDigitalFactory $certificadoDigitalFactory, $entityManager, $fileUploader, $environment, $fileServerLink) {
        $this->certificadoDigitalFactory = $certificadoDigitalFactory;
        $this->connection = $entityManager->getConnection();
        $this->fileUploader  = $fileUploader;
        $this->environment   = $environment;
        $this->fileServerLink = $fileServerLink;
    }
    
    public function getInscrMunicip($cdinscmuni, $cdempresa, $nrorg, $getLink = false) {
        $inscrMunicip = $this->certificadoDigitalFactory->getInscrMunicip($cdinscmuni, $cdempresa, $nrorg)[0];
        if($inscrMunicip['DSDIRCERTIFICADO']) {
            $inscrMunicip['DSDIRCERTARRAY'] = $this->getArrayFiles($inscrMunicip['DSDIRCERTIFICADO'], $getLink,  'prod');
        }
        return $inscrMunicip;
    }
    
    public function updateDsDirCert($cdinscmuni, $cdempresa, $nrorg, $dsdircert) {
        $this->certificadoDigitalFactory->updateDsDirCert($cdinscmuni, $cdempresa, $nrorg, $dsdircert);
    }
    
    public function getArrayFiles($filesString, $getLink = false, $mode = 'prod') {
        $filesArray = array();
        $array = array_filter(explode('<', $filesString));
        $count = 0;
        foreach ($array as $element) {
            $element = explode('>', $element);
            $file = array("fileName" => $element[0]);
            if($getLink) $file["path"] = $this->fileServerLink.$element[1];
            array_push($filesArray, $file);
            $count++;
        }
        return $filesArray;
    }
    
    public function createFileDataSet($inscrMunicip) {
        $arrayReturn       = array();
        if(isset($inscrMunicip['DSDIRCERTARRAY'])) {
            foreach($inscrMunicip['DSDIRCERTARRAY'] as $file) {
                $file['CDEMPRESA']  = $inscrMunicip['CDEMPRESA'];
                $file['CDINSCMUNI'] = $inscrMunicip['CDINSCMUNI'];
                $file['NRORG']      = $inscrMunicip['NRORG'];
                $file['dir']        = '';
                array_push($arrayReturn, $file);
            }
        }
        return $arrayReturn;
    }
    
    public function factoryFromFileData($cdinscmuni, $empresa, $nrorg, array $fileData, $destinationFolder) {
        list($contentType, $base64String) = explode(',', $fileData['b64File']);
        try {
            if(!is_dir($destinationFolder)) {
                mkdir($destinationFolder, 0777, true);
            }
        }
        catch (\Exception $e) {
            throw new\Exception('Não foi possível salvar o certificado. Verifique suas permissões!', 100);
        }
        
        $filePath = $destinationFolder.DIRECTORY_SEPARATOR.$fileData['name'];
        $file     = base64_decode($base64String);

        $this->generateAndSavePemLocal($cdinscmuni, $empresa, $nrorg, $file, $destinationFolder);
                
        file_put_contents($filePath, base64_decode($base64String));
        chmod($filePath, 0777);

        return new UploadedFile($filePath, $fileData['name'], $contentType, $fileData['size'], null, true);
    }
    
    public function saveCertificadoFileServer($row) {
        $empresa      = $row['CDEMPRESA'];
        $cdinscmuni   = $row['CDINSCMUNI'];
        $nrorg        = isset($row['NRORG']) ? $row["NRORG"] : $this->environment->getNrorg();

        try {
            $inscrMunicip = $this->getInscrMunicip($cdinscmuni, $empresa, $nrorg);
            
            $files        = $row['CERTIFICADO'];
            $dsDirCert    = (isset($inscrMunicip['DSDIRCERTIFICADO']))      ? $inscrMunicip['DSDIRCERTIFICADO'] : '';
            $arrayCert    = (isset($inscrMunicip['DSDIRCERTARRAY'])) ? $inscrMunicip['DSDIRCERTARRAY'] : array();
            if(count($arrayCert) >= 1) throw new\Exception('Já existe um certificado para essa inscrição. Exclua e tente novamente');
            foreach ($files as $file) {
                $dsDirPem  = $this->generateAndSavePemFileServer($cdinscmuni, $empresa, $nrorg, $file);
                $fileName  = $file['name'];
                $extension = explode(".", $fileName);
                $extension = end($extension);
               
                if(($extension !== self::EXTENSION_PFX) && ($extension !== self::EXTENSION_P12)) {
                    throw \Exception($fileName.': extensão do arquivo não aceita.');
                }
                else {
                    $dsDirCert.='<'.$file['name'].'>';
                    $dsDirCert.=$file["path"];
                    $dsDirPem  ='<certKey.pem>'.$dsDirPem;
                    $dsDirCert.=$dsDirPem;
                    $this->updateDsDirCert($cdinscmuni, $empresa, $nrorg, $dsDirCert);
                }
            }
        }
        catch(\Exception $e) {
            throw $e;
        }
    }
    
    private function generateAndSavePemLocal($cdinscmuni, $empresa, $nrorg, $data, $destinationFolder) {
        $senha = Util::getSenhaCertificadoDigital($this->connection, $empresa, $cdinscmuni, $nrorg);
      

        openssl_pkcs12_read($data, $cert, $senha);
        $filePath = $destinationFolder.DIRECTORY_SEPARATOR.'certKey.pem';
        file_put_contents($filePath, $cert["pkey"]."\r\n".$cert["cert"]);
        
        chmod($filePath, 0777);
    }
    
    private function generateAndSavePemFileServer($cdinscmuni, $empresa, $nrorg, $file, $mode = 'prod') {
        $senha = Util::getSenhaCertificadoDigital($this->connection, $empresa, $cdinscmuni, $nrorg);
        
        $path = $file["path"];
        $path = $this->fileServerLink.$path;
        $data = file_get_contents($path);

        if (!openssl_pkcs12_read($data, $cert, $senha)) throw new \Exception ('A senha do certificado está inválida!', 101);

        $tmpFile = sys_get_temp_dir().DIRECTORY_SEPARATOR.'certKey'.uniqid().'.pem';
        file_put_contents($tmpFile, $cert["pkey"]."\r\n".$cert["cert"]);
        $certKey = file_get_contents($tmpFile);
        $pem = base64_encode($certKey);

        $pem = 'data: '.mime_content_type($tmpFile).';base64,'.$pem;
        
        $lastModify = filectime($tmpFile);

        $filePem = array(
            "name" => "certKey.pem", 
            "lastModified" => $lastModify,
            "lastModifiedDate" => '',
            "webkitRelativePath" => '',
            "type" => '',
            "size"  => filesize($tmpFile),
            "b64File" => $pem);
            
        $row['CERTKEY']    = array();
        $row['CERTKEY'][0] = $filePem;
        $this->fileUploader->uploadFilesInRow($row, 'CERTKEY', 'Certificados');
        unlink($tmpFile);
        return $row['CERTKEY'][0]['path'];
    }
    
    public function deleteFileFileServer($row) {
        $empresa      = $row['CDEMPRESA'];
        $cdinscmuni   = $row['CDINSCMUNI'];
        $nrorg        = $row['NRORG'];
        
        $this->updateDsDirCert($cdinscmuni, $empresa, $nrorg, '');
    }
}
