<?php

namespace Teknisa\DF\Services;


use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO\Request\Row;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\Masks\Masks;
use Teknisa\Libs\Util\NewCode;
use Teknisa\Libs\Util\Utilities;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
    
use Teknisa\DF\Factories\Ger40100IntegracaoViaEmailFactory;
use Teknisa\DF\Modules\Entrada\Factories\Dfe07900ControleDeArquivosXmlFactory;
use Teknisa\DF\Modules\Entrada\Services\Cripto;
use Teknisa\DF\Services\CallServiceIntegraViaEmail;

class Ger40100IntegracaoViaEmailService {
    protected $ger40100IntegracaoViaEmailFactory;
    protected $environment;
    protected $entityManager;
    protected $cripto;
    protected $dfe07900ControleDeArquivosXmlFactory;
    protected $callServiceIntegraViaEmail;
    protected $erros = [];
    

    public function __construct(Ger40100IntegracaoViaEmailFactory $ger40100IntegracaoViaEmailFactory, $entityManager, $environment, $cripto, Dfe07900ControleDeArquivosXmlFactory $dfe07900ControleDeArquivosXmlFactory, CallServiceIntegraViaEmail $callServiceIntegraViaEmail) {
        $this->ger40100IntegracaoViaEmailFactory = $ger40100IntegracaoViaEmailFactory;
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->cripto = $cripto;
        $this->dfe07900ControleDeArquivosXmlFactory = $dfe07900ControleDeArquivosXmlFactory;
        $this->callServiceIntegraViaEmail = $callServiceIntegraViaEmail;
        set_time_limit(0);
    }

    
    public function saveIntegracaoViaEmail($row){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try{
            $verificaIntg =  $this->ger40100IntegracaoViaEmailFactory->verificaIntegracaoViaEmail($row['DSSERVEMAIL'], $row['DSLOGINEMAIL']);
            $boolSenhaAlterada = $row['senhaSMTPAlterada'];
            if($boolSenhaAlterada){
                $valorDescriptografado = $row['CDSENHAEMAIL'];
                $valorCriptografado = $this->cripto->cripto($valorDescriptografado,2);
                $row->set('CDSENHAEMAIL', $valorCriptografado);
            }
            
            if($verificaIntg["NRREGS"] == 0) {
                $this->ger40100IntegracaoViaEmailFactory->insereIntgViaEmail($row['DSSERVEMAIL'], $row['DSLOGINEMAIL'], $row['CDSENHAEMAIL'], $row['NRPORTAEMAIL'], $row['IDUTTLSEMAIL'], $row['IDUTSSLEMAIL'], $row['NRPERIODEMAIL'], $row['DSARQEXTENSAO'], $row['DIRPHP'], $row['JENKINSPATH'], $row['CALLSERVPATH']);
            }
            else {
                $this->ger40100IntegracaoViaEmailFactory->alteraIntgViaEmail($row['DSSERVEMAIL'], $row['DSLOGINEMAIL'], $row['CDSENHAEMAIL'], $row['NRPORTAEMAIL'], $row['IDUTTLSEMAIL'], $row['IDUTSSLEMAIL'], $row['NRPERIODEMAIL'], $row['DSARQEXTENSAO'], $row['DIRPHP'], $row['JENKINSPATH'], $row['CALLSERVPATH']);
            }
            $connection->commit();
        }catch(\Exception $e){
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function verificaEmail($row){
        try{
            if($this->environment->getNrOrgTrab() != 1) {
                $caminhoDir = "SaasHosting";
            }
            else {
                $caminhoDir =  $this->dfe07900ControleDeArquivosXmlFactory->getDiretorioImportacao();
            }
            $valorDescriptografado = $this->cripto->unCripto($row['CDSENHAEMAIL'],2);
            $row->set('CDSENHAEMAIL', $valorDescriptografado);
            $retorno[0] = $this->callServiceIntegraViaEmail->VerificaEmail($row, $caminhoDir);
        } catch(\Exception $e){
            $this->entityManager->rollback();
            throw $e;
        }
        return $retorno;
    }
    
    public function verificaTodosEmails($row){
        try{
            var_dump('Inicio Integração por Email.');
            $caminhoDir =  $this->dfe07900ControleDeArquivosXmlFactory->getDiretorioImportacao();
            $integraEmailALL =  $this->ger40100IntegracaoViaEmailFactory->retornaTodosEmails($row);
            for($ind=0; $ind<count($integraEmailALL); $ind++){
                var_dump($integraEmailALL[$ind]['DSLOGINEMAIL']);
                $valorDescriptografado = $this->cripto->unCripto($integraEmailALL[$ind]['CDSENHAEMAIL'],2);
                $integraEmailALL[$ind]['CDSENHAEMAIL'] = $valorDescriptografado;
                $retorno[$ind] = $this->callServiceIntegraViaEmail->VerificaEmail($integraEmailALL[$ind], $caminhoDir);
            }
            var_dump('Fim Integração por Email.');
            //die();
        } catch(\Exception $e){
            $this->entityManager->rollback();
            throw $e;
        }
        return $retorno;
    }
    
    public function setAgendamentoIntegracaoEmail($nrorg, $useVpd, $newEntityManager, $newFileUploader) {
        if($useVpd) $this->entityManager = $newEntityManager;
        
        $this->environment->setNrOrg($nrorg);
        $this->environment->setNrOrgTrab($nrorg);
        $this->environment->setCurrentUserId('AUTOMATICO');
        $this->ger40100IntegracaoViaEmailFactory->setAgendamentoIntegracaoEmail($nrorg, $useVpd, $newEntityManager, $newFileUploader);
    }
}