<?php

namespace Teknisa\DF\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\Environment;
use Teknisa\DF\Factories\ParametrizacaoFiscalCentralizadaFactory;

use Teknisa\Model\Fiscal\Entities\Msgfiscregra;
use Teknisa\Model\Fiscal\Entities\Msgfiscncm;
use Teknisa\Model\Fiscal\Entities\Msgfiscprod;
use Teknisa\Model\Fiscal\Entities\Dadosfiscregra;
use Teknisa\Model\Financeiro\Entities\Dadosfiscncm;
use Teknisa\Model\Fiscal\Entities\Dadosfiscprod;

use Teknisa\Model\Fiscal\Entities\Msgcfopadrao;
use Teknisa\Model\Fiscal\Entities\Msgcfopncm;
use Teknisa\Model\Fiscal\Entities\Msgcfopregfisc;
use Teknisa\Model\Fiscal\Entities\Cfopadraoregfis;
use Teknisa\Model\Fiscal\Entities\Cfopncmregrafis;
use Teknisa\Model\Fiscal\Entities\Cfopregrafiscal;

class ParametrizacaoFiscalCentralizadaService {
    protected $entityManager;
    protected $environment;
    protected $paramCentFactory;
    protected $crudService;

    public function __construct(EntityManager $entityManager,
                                Environment $environment,
                                ParametrizacaoFiscalCentralizadaFactory $paramCentFactory,
                                $crudService)
    {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->paramCentFactory = $paramCentFactory;
        $this->crudService = $crudService;
    }

    public function deletarRegraImpostoPadrao($cdregrafisc, $cdimposto, $nrseqdadosfisc){
        $regraImposto = $this->entityManager->getRepository('Teknisa\Model\Fiscal\Entities\Dadosfiscregra')->findOneBy(
                array('cdregrafisc' => $cdregrafisc, 'cdimposto' => $cdimposto, 'nrseqdadosfisc' => $nrseqdadosfisc));
        $this->removerEntidades($regraImposto);
    }

    public function deletarRegraImpostoNCM($cdregrafisc, $cdimposto, $nrseqdadosfisc, $cdclasfisc){
        $regraImposto = $this->entityManager->getRepository('Teknisa\Model\Financeiro\Entities\Dadosfiscncm')->findOneBy(
                array('cdregrafisc' => $cdregrafisc, 'cdimposto' => $cdimposto, 'nrseqdadosfisc' => $nrseqdadosfisc, 'cdclasfisc' => $cdclasfisc));
        $this->removerEntidades($regraImposto);
    }

    public function deletarRegraImpostoProduto($cdregrafisc, $cdimposto, $nrseqdadosfisc, $cdproduto){
        $regraImposto = $this->entityManager->getRepository('Teknisa\Model\Fiscal\Entities\Dadosfiscprod')->findOneBy(
                array('cdregrafisc' => $cdregrafisc, 'cdimposto' => $cdimposto, 'nrseqdadosfisc' => $nrseqdadosfisc, 'cdproduto' => $cdproduto));
        $this->removerEntidades($regraImposto);
    }

    public function deletarMensagensImpostoPadrao($cdregrafisc, $cdimposto, $nrseqdadosfisc){
        $mensagens = $this->entityManager->getRepository('Teknisa\Model\Fiscal\Entities\Msgfiscregra')->findBy(
                array('cdregrafisc' => $cdregrafisc, 'cdimposto' => $cdimposto, 'nrseqdadosfisc' => $nrseqdadosfisc));
        $this->removerEntidades($mensagens);
    }

    public function deletarMensagensImpostoNCM($cdregrafisc, $cdimposto, $nrseqdadosfisc, $cdclasfisc){
        $mensagens = $this->entityManager->getRepository('Teknisa\Model\Fiscal\Entities\Msgfiscncm')->findBy(
                array('cdregrafisc' => $cdregrafisc, 'cdimposto' => $cdimposto, 'nrseqdadosfisc' => $nrseqdadosfisc, 'cdclasfisc' => $cdclasfisc));
        $this->removerEntidades($mensagens);
    }

    public function deletarMensagensImpostoProduto($cdregrafisc, $cdimposto, $nrseqdadosfisc, $cdproduto){
        $mensagens = $this->entityManager->getRepository('Teknisa\Model\Fiscal\Entities\Msgfiscprod')->findBy(
                array('cdregrafisc' => $cdregrafisc, 'cdimposto' => $cdimposto, 'nrseqdadosfisc' => $nrseqdadosfisc, 'cdproduto' => $cdproduto));
        $this->removerEntidades($mensagens);
    }

    public function deletarRegraCfopPadrao($cdregrafisc, $idcfopinterest){
        $regraImposto = $this->entityManager->getRepository('Teknisa\Model\Fiscal\Entities\Cfopadraoregfis')->findOneBy(
                array('cdregrafisc' => $cdregrafisc, 'idcfopinterest' => $idcfopinterest));
        $this->removerEntidades($regraImposto);
    }

    public function deletarRegraCfopNCM($cdregrafisc, $idcfopinterest, $cdclasfisc){
        $regraImposto = $this->entityManager->getRepository('Teknisa\Model\Fiscal\Entities\Cfopncmregrafis')->findOneBy(
                array('cdregrafisc' => $cdregrafisc, 'idcfopinterest' => $idcfopinterest, 'cdclasfisc' => $cdclasfisc));
        $this->removerEntidades($regraImposto);
    }

    public function deletarRegraCfopProduto($cdregrafisc, $idcfopinterest, $cdproduto){
        $regraImposto = $this->entityManager->getRepository('Teknisa\Model\Fiscal\Entities\Cfopregrafiscal')->findOneBy(
                array('cdregrafisc' => $cdregrafisc, 'idcfopinterest' => $idcfopinterest, 'cdproduto' => $cdproduto));
        $this->removerEntidades($regraImposto);
    }

    public function deletarMensagensCfopPadrao($cdregrafisc, $idcfopinterest){
        $mensagens = $this->entityManager->getRepository('Teknisa\Model\Fiscal\Entities\Msgcfopadrao')->findBy(
                array('cdregrafisc' => $cdregrafisc, 'idcfopinterest' => $idcfopinterest));
        $this->removerEntidades($mensagens);
    }

    public function deletarMensagensCfopNCM($cdregrafisc, $idcfopinterest, $cdclasfisc){
        $mensagens = $this->entityManager->getRepository('Teknisa\Model\Fiscal\Entities\Msgcfopncm')->findBy(
                array('cdregrafisc' => $cdregrafisc, 'idcfopinterest' => $idcfopinterest, 'cdclasfisc' => $cdclasfisc));
        $this->removerEntidades($mensagens);
    }

    public function deletarMensagensCfopProduto($cdregrafisc, $idcfopinterest, $cdproduto){
        $mensagens = $this->entityManager->getRepository('Teknisa\Model\Fiscal\Entities\Msgcfopregfisc')->findBy(
                array('cdregrafisc' => $cdregrafisc, 'idcfopinterest' => $idcfopinterest, 'cdproduto' => $cdproduto));
        $this->removerEntidades($mensagens);
    }

    public function consultarErrosRegra($cdregrafisc){
        return $erroDadosFinanceiros = $this->paramCentFactory->consultarErrosRegra($cdregrafisc);
    }
    
    public function getNewCodeRegra() {
        return $this->paramCentFactory->getNewCodeRegra($this->environment->getNrorgTrab());
    }
    
    public function insereImpRegra($requestSalvar) {
        $saveRows = $this->crudService->getRowsToSave($requestSalvar, 'impregrafiscal');
        return $this->crudService->save('impregrafiscal', $saveRows);
    }
    
    public function saveRegra($requestSalvar) {
        $saveRows = $this->crudService->getRowsToSave($requestSalvar, 'regrafiscal');
        return $this->crudService->save('regrafiscal', $saveRows);
    }
    
    public function consultaRepetida($row) {
        return $this->paramCentFactory->consultaRepetida($row['CDEMPRESA'],$row['IDENTRSAIDOP'],$row['CDTIPOOPER'],$row['CDREGRAFISC']);
    }
    
    public function getImpostosGrupo($cdgrupoimpost) {
        return $this->paramCentFactory->getImpostosGrupo($cdgrupoimpost, $this->environment->getNrorgTrab());
    }
    
    public function getImpostosGrupoAntigo($cdregrafisc) {
        return $this->paramCentFactory->getImpostosGrupoAntigo($cdregrafisc, $this->environment->getNrorgTrab());
    }

    private function removerEntidades($entidades){
        if(is_array($entidades)){
            foreach($entidades as $ent){
                $this->entityManager->remove($ent);
            }
        }
        else{
            $this->entityManager->remove($entidades);
        }
        $this->entityManager->flush();
    }
    
    public function saveAgrupImposto($cdRegraFisc, $cdGrupImposto) {
        return $this->paramCentFactory->saveAgrupImposto($cdRegraFisc, $cdGrupImposto);
    }
    
    public function validaProdutoRepitidoRegra($cdregrafisc, $cdpaisdest, $sgestadest, $cdpaisorig, $sgestaorig, $idtipodestfisc, $idtipoorigfisc, $idutufdest, $idutuforig, $cdproduto, $cdimposto) {
        return $this->paramCentFactory->validaProdutoRepitidoRegra($cdregrafisc, $cdpaisdest, $sgestadest, $cdpaisorig, $sgestaorig, $idtipodestfisc, $idtipoorigfisc, $idutufdest, $idutuforig, $cdproduto, $cdimposto);
    }
    
    public function getMaxdadosfiscprod($cdregrafisc, $cdimposto) {
        return $this->paramCentFactory->getMaxdadosfiscprod($cdregrafisc, $cdimposto);
    }
    
    public function getMaxdadosfiscncm($cdregrafisc, $cdimposto) {
        return $this->paramCentFactory->getMaxdadosfiscncm($cdregrafisc, $cdimposto);
    }
    
    public function saveReplicacaoRegraFisc($cdregrafisc, $cdregrafiscorigem, $validadeInicial) {
        
        $connection = $this->entityManager->getConnection();
        
        try{
            $connection->beginTransaction();
            
            
                $this->paramCentFactory->saveRegraFiscReplicacaoRegraFisc($cdregrafisc, $cdregrafiscorigem, $validadeInicial);
                
                $this->paramCentFactory->saveImpostoReplicacaoRegraFisc($cdregrafisc, $cdregrafiscorigem);
                
                $this->paramCentFactory->saveImpPadraoReplicacaoRegraFisc($cdregrafisc, $cdregrafiscorigem);
                $this->paramCentFactory->saveMsgImpPadraoReplicacaoRegraFisc($cdregrafisc, $cdregrafiscorigem);
                
                $this->paramCentFactory->saveImpNCMReplicacaoRegraFisc($cdregrafisc, $cdregrafiscorigem);
                $this->paramCentFactory->saveMsgImpNCMReplicacaoRegraFisc($cdregrafisc, $cdregrafiscorigem);
                
                $this->paramCentFactory->saveImpProdReplicacaoRegraFisc($cdregrafisc, $cdregrafiscorigem);
                $this->paramCentFactory->saveMsgImpProdReplicacaoRegraFisc($cdregrafisc, $cdregrafiscorigem);
                
                $this->paramCentFactory->saveCfopPadraoReplicacaoRegraFisc($cdregrafisc, $cdregrafiscorigem);
                $this->paramCentFactory->saveMsgCfopPadraoReplicacaoRegraFisc($cdregrafisc, $cdregrafiscorigem);
                
                $this->paramCentFactory->saveCfopNCMReplicacaoRegraFisc($cdregrafisc, $cdregrafiscorigem);
                $this->paramCentFactory->saveMsgCfopNCMoReplicacaoRegraFisc($cdregrafisc, $cdregrafiscorigem);
                
                $this->paramCentFactory->saveCfopProdReplicacaoRegraFisc($cdregrafisc, $cdregrafiscorigem);
                $this->paramCentFactory->saveMsgCfopProdReplicacaoRegraFisc($cdregrafisc, $cdregrafiscorigem);
                
                $this->paramCentFactory->saveGrupoFiscReplicacaoRegraFisc($cdregrafisc, $cdregrafiscorigem);
                $this->paramCentFactory->saveMsgReplicacaoRegraFisc($cdregrafisc, $cdregrafiscorigem);
            
            $connection->commit();
            
        }catch(\Exception $e){
            
            $connection->rollback();
            
            throw new \Exception($e->getMessage());
        }
        
        
    }
    
    public function saveVinculoReplicacaoRegraFisc($cdregrafisc, $cdregrafiscorigem, $validadeFinal) {
        
        $connection = $this->entityManager->getConnection();
        
        try{
            $connection->beginTransaction();
            
            $this->paramCentFactory->saveVinculoReplicacaoRegraFisc($cdregrafisc, $cdregrafiscorigem, $validadeFinal);
            
            $connection->commit();
            
        }catch(\Exception $e){
            
            $connection->rollback();
            
            throw new \Exception($e->getMessage());
        }
        
        
    }
    
    
}